//Problema P51080 del jutge

#include <cstdlib>
#include <iostream>

using namespace std;

template <class T> class Arbre { 

private:     

  struct node_arbre {
    T info;
    node_arbre* segE;
    node_arbre* segD;
  };

  node_arbre* primer_node;

  // especificació operacions privades

  node_arbre* copia_node_arbre(node_arbre* m) { 
    /* Pre: cert */
    /* Post: el resultat és NULL si m és NULL; 
       en cas contrari, el resultat apunta al node arrel 
       d'una jerarquia de nodes que és una còpia de la 
       jerarquia de nodes que té el node apuntat per m com a arrel */
    node_arbre* n;
    if (m==NULL) n=NULL;
    else {
      n = new node_arbre;
      n->info = m->info;
      n->segE = copia_node_arbre(m->segE);
      n->segD = copia_node_arbre(m->segD);
    }
    return n;
  }

  void esborra_node_arbre(node_arbre* m) {  
    /* Pre: cert */
    /* Post no fa res si m és NULL; en cas contrari, 
       allibera espai de tots els nodes de la 
       jerarquia que té el node apuntat per m com a arrel */
    if (m != NULL) {
      esborra_node_arbre(m->segE);
      esborra_node_arbre(m->segD);
      delete m;
    }
  }

public:

  // especificació operacions públiques

  Arbre() {
    /* Pre: cert */
    /* Post: el p.i. és un arbre buit */
    primer_node= NULL;
  }

  Arbre(const Arbre& original) {
    /* Pre: cert */
    /* Post: el p.i. és una còpia d'original */
    //if (this != &original)     
      primer_node = copia_node_arbre(original.primer_node);
    //else
    //  primer_node = NULL;
  }

  ~Arbre() {
    esborra_node_arbre(primer_node);
  }

  Arbre& operator=(const Arbre& original) {
    if (this != &original) {
      esborra_node_arbre(primer_node);
      primer_node = copia_node_arbre(original.primer_node);
    }
    return *this;
  }

  void a_buit() {
    /* Pre: cert */
    /* Post: el p.i. és un arbre buit */
    esborra_node_arbre(primer_node);
    primer_node= NULL;
  }        

  void plantar(const T &x, Arbre &a1, Arbre &a2) {
    /* Pre: el p.i. és buit, a1=A1, a2=A2 */
    /* Post: el p.i. té x com a arrel, 
       A1 com a fill esquerre i A2 com a fill dret; 
       a1 i a2 són buits */
    if (this != &a1 and this != &a2) {
      if (primer_node==NULL) {
        node_arbre* aux= new node_arbre;
        aux->info= x;
        aux->segE= a1.primer_node;
        if (a1.primer_node == a2.primer_node) aux->segD= copia_node_arbre(a1.primer_node);
        else  aux->segD= a2.primer_node;
        primer_node= aux;
        a1.primer_node= NULL;
        a2.primer_node= NULL;
      }
    }
  }


  void fills (Arbre &fe, Arbre &fd) {
    /* Pre: el p.i. no és buit i li diem A, fe i fd són buits */
    /* Post: fe és el fill esquerre d'A, fd és el fill dret d'A,
       el p.i. és buit */
    if (primer_node!=NULL and fe.primer_node==NULL
        and fd.primer_node==NULL) {
      if (&fe != &fd) {       
        node_arbre* aux;
        aux= primer_node;
        fe.primer_node= aux->segE;
        fd.primer_node= aux->segD;
        primer_node= NULL;
        delete aux;
      }
    }
  }

  T arrel() const {
    /* Pre: el p.i. no és buit */
    /* Post: el resultat és l'arrel del p.i. */
    if (primer_node!=NULL)
      return primer_node->info;    
  }

  bool es_buit() const {
    /* Pre: cert */
    /* Post: el resultat indica si el p.i. és un arbre buit */
    return (primer_node==NULL);
  }


  int cami() const {
  /* Pre: cert */
  /* Post: retorna la longitud del camí més llarg d'A */
    return ecami(primer_node).first;
  }

private:

  pair<int,int> ecami(node_arbre* n) const {
    if (n==NULL) return make_pair(0,0);
    else{
      pair<int,int> c1=ecami(n->segE);
      pair<int,int> c2=ecami(n->segD);
      return make_pair(max(c1.first,max(c2.first,c1.second+c2.second+1)),
		  max(c1.second,c2.second)+1);
    }
  }

};

void llegir_arbre_int(Arbre<int>& a, int marca){
/* Pre: a es buit; el canal estandar d'entrada cont� el recorregut en preordre
   d'un arbre binari A d'enters marcat amb "marca" */  
/* Post: a = A */
  Arbre<int> a1;
  Arbre<int> a2;
  int x;
  cin >> x;
  if (x!= marca) {
    llegir_arbre_int(a1, marca);
    llegir_arbre_int(a2, marca);
    a.plantar(x,a1,a2);
  }
}

int main(){

  int n;
  cin >> n;
  for (int i=0; i<n; ++i){
    Arbre<int> a;
    llegir_arbre_int(a,-1);
    cout <<  a.cami() << endl;
  }
}

